<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<!--
	bil.xsl
	**************************************************
	This style sheet can be used to generate an AppBuilder module style sheet from a given window XML file.

	To Do
	* layoutalign attribute must be implemented
-->

<!--
	HEADER AND FOOTER
	**************************************************
	This template creates an XML and AppBuilder header and footer.
-->
<xsl:template match="text()|@*"/>
<xsl:output method="xml" encoding="US-ASCII"/>

<xsl:template match="/">
<xsl:element name="xsl:stylesheet">
<xsl:attribute name="xmlns:xsl">http://www.w3.org/1999/XSL/Transform</xsl:attribute><xsl:text>

</xsl:text><xsl:element name="xsl:template"><xsl:attribute name="match">text()|@*</xsl:attribute></xsl:element><xsl:text>
</xsl:text><xsl:element name="xsl:output"><xsl:attribute name="method">text</xsl:attribute></xsl:element><xsl:text>

</xsl:text><xsl:element name="xsl:template">
<xsl:attribute name="match">/codeframe</xsl:attribute>:bil-version	1 0
<xsl:apply-templates/>
</xsl:element><xsl:text>
</xsl:text></xsl:element>
<!--<xsl:text disable-output-escaping="yes">&lt;/xsl:stylesheet&gt;</xsl:text>--></xsl:template>

<!--
	GENERAL TEMPLATES
	**************************************************
	These are general templates to be called from within other templates.
-->
<!--
	insertheader
	************
	Creates a standard ":element" header with an id and an opening bracket.
-->
<xsl:template name="insertheader">
:element <xsl:value-of select="@id"/>
(</xsl:template>

<!--
	precedingname
	*************
	Inserts (:point 0 0) when first non-menu element, or (:obj @id 0) otherwise.
-->
<xsl:template name="precedingname"><xsl:with-param name="position"/>
<xsl:choose><xsl:when test="../*[position() = $position]/@id != @id and (local-name(../*[position() = $position]) != 'menubar' and local-name(../*[position() = $position]) != 'popupmenu')">(:obj <xsl:value-of select="../*[position() = $position]/@id"/><xsl:if test="../*[position() = $position]/@popup">_popup</xsl:if> 0)</xsl:when>
<xsl:when test="$position = 1">(:point 0 0)</xsl:when>
<xsl:otherwise><xsl:call-template name="precedingname"><xsl:with-param name="position" select="$position - 1"/></xsl:call-template></xsl:otherwise></xsl:choose>
</xsl:template>

<!--
	followingname
	*************
	Inserts (:point 0 0) when last non-menu element, or (:obj @id 0) otherwise.
-->
<xsl:template name="followingname"><xsl:with-param name="position"/><xsl:message><xsl:value-of select="@id"/>: <xsl:value-of select="$position"/></xsl:message>
<xsl:choose><xsl:when test="../*[position() = $position]/@id != @id and (local-name(../*[position() = $position]) != 'menubar' and local-name(../*[position() = $position]) != 'popupmenu')">(:obj <xsl:value-of select="../*[position() = $position]/@id"/><xsl:if test="../*[position() = $position]/@popup">_popup</xsl:if> 0)</xsl:when>
<xsl:when test="$position = last()">(:point 0 0)</xsl:when>
<xsl:otherwise><xsl:call-template name="followingname"><xsl:with-param name="position" select="$position + 1"/></xsl:call-template></xsl:otherwise></xsl:choose>
</xsl:template>

<!--
	insertattachments
	*****************
	Inserts the appropriate attachments according to the selected layout.
-->
<xsl:template name="insertattachments">
<xsl:choose>
<xsl:when test="../@layout='horizontal'"><!-- Horizontal layout -->
	:north-attachment	(:center-grid-line 50 0)
	<xsl:choose><xsl:when test="../@layoutalign='left'">:west-attachment	<xsl:call-template name="precedingname"><xsl:with-param name="position" select="position()"/></xsl:call-template></xsl:when>
	<xsl:when test="../@layoutalign='right'">:east-attachment	<xsl:call-template name="followingname"><xsl:with-param name="position" select="position()"/></xsl:call-template></xsl:when>
	<xsl:otherwise>:west-attachment	<xsl:call-template name="precedingname"><xsl:with-param name="position" select="position()"/></xsl:call-template></xsl:otherwise></xsl:choose>
</xsl:when>
<xsl:when test="../@layout='vertical'"><!-- Vertical layout -->
	:north-attachment	<xsl:call-template name="precedingname"><xsl:with-param name="position" select="position()"/></xsl:call-template>
	:west-attachment	(:center-grid-line 50 0)
</xsl:when>
<xsl:otherwise><!-- Border layout -->
<xsl:choose><xsl:when test="local-name()='panel'"><xsl:call-template name="insertpanelattachments"><xsl:with-param name="position" select="@position"/></xsl:call-template></xsl:when>
<xsl:otherwise>	:north-attachment	(:point 0 0)
	:south-attachment	(:point 0 0)
	:west-attachment	(:point 0 0)
	:east-attachment	(:point 0 0)</xsl:otherwise></xsl:choose>
</xsl:otherwise>
</xsl:choose>
</xsl:template>

<!--
	insertpanelattachments
	**********************
	Inserts the appropriate attachments for a panel according to the selected layout and position.
	Call this template with a parameter 'position', which holds the position.
-->
<xsl:template name="insertpanelattachments"><xsl:with-param name="position"/>
<xsl:choose><xsl:when test="local-name()='panel'">
<xsl:if test="$position='top'">	:north-attachment	(:point 0 0)
	:west-attachment	(:point 0 0)
	:east-attachment	(:point 0 0)</xsl:if>
<xsl:if test="$position='bottom'">	:south-attachment	(:point 0 0)
	:west-attachment	(:point 0 0)
	:east-attachment	(:point 0 0)</xsl:if>
<xsl:if test="$position='left'">	:north-attachment	<xsl:choose><xsl:when test="../*[@position='top'] and not(../panel[@position='top'])">(:obj <xsl:value-of select="@id"/>_top 0)</xsl:when><xsl:when test="../panel[@position='top']">(:obj <xsl:value-of select="../panel[@position='top']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose>
	:south-attachment	<xsl:choose><xsl:when test="../*[@position='bottom'] and not(../panel[@position='bottom'])">(:obj <xsl:value-of select="@id"/>_bottom 0)</xsl:when><xsl:when test="../panel[@position='bottom']">(:obj <xsl:value-of select="../panel[@position='bottom']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose>
	:west-attachment	(:point 0 0)</xsl:if>
<xsl:if test="$position='right'">	:north-attachment	<xsl:choose><xsl:when test="../*[@position='top'] and not(../panel[@position='top'])">(:obj <xsl:value-of select="@id"/>_top 0)</xsl:when><xsl:when test="../panel[@position='top']">(:obj <xsl:value-of select="../panel[@position='top']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose>
	:south-attachment	<xsl:choose><xsl:when test="../*[@position='bottom'] and not(../panel[@position='bottom'])">(:obj <xsl:value-of select="@id"/>_bottom 0)</xsl:when><xsl:when test="../panel[@position='bottom']">(:obj <xsl:value-of select="../panel[@position='bottom']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose>
	:east-attachment	(:point 0 0)</xsl:if>
<xsl:if test="$position='center'">	:north-attachment	<xsl:choose><xsl:when test="../*[@position='top'] and not(../panel[@position='top'])">(:obj <xsl:value-of select="@id"/>_top 0)</xsl:when><xsl:when test="../panel[@position='top']">(:obj <xsl:value-of select="../panel[@position='top']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose>
	:south-attachment	<xsl:choose><xsl:when test="../*[@position='bottom'] and not(../panel[@position='bottom'])">(:obj <xsl:value-of select="@id"/>_bottom 0)</xsl:when><xsl:when test="../panel[@position='bottom']">(:obj <xsl:value-of select="../panel[@position='bottom']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose>
	:west-attachment	<xsl:choose><xsl:when test="../*[@position='left'] and not(../panel[@position='left'])">(:obj <xsl:value-of select="@id"/>_left 0)</xsl:when><xsl:when test="../panel[@position='left']">(:obj <xsl:value-of select="../panel[@position='left']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose>
	:east-attachment	<xsl:choose><xsl:when test="../*[@position='right'] and not(../panel[@position='right'])">(:obj <xsl:value-of select="@id"/>_right 0)</xsl:when><xsl:when test="../panel[@position='right']">(:obj <xsl:value-of select="../panel[@position='right']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose></xsl:if>
<xsl:if test="$position='middle'">	:north-attachment	(:center-grid-line 50 0)
	:west-attachment	(:center-grid-line 50 0)</xsl:if>
</xsl:when><xsl:otherwise>
<xsl:if test="$position='top'">	:north-attachment	(:point 0 0)
	:west-attachment	(:point 0 0)
	:east-attachment	(:point 0 0)</xsl:if>
<xsl:if test="$position='bottom'">	:south-attachment	(:point 0 0)
	:west-attachment	(:point 0 0)
	:east-attachment	(:point 0 0)</xsl:if>
<xsl:if test="$position='left'">	:north-attachment	<xsl:choose><xsl:when test="*[@position='top'] and not(panel[@position='top'])">(:obj <xsl:value-of select="@id"/>_top 0)</xsl:when><xsl:when test="panel[@position='top']">(:obj <xsl:value-of select="panel[@position='top']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose>
	:south-attachment	<xsl:choose><xsl:when test="*[@position='bottom'] and not(panel[@position='bottom'])">(:obj <xsl:value-of select="@id"/>_bottom 0)</xsl:when><xsl:when test="panel[@position='bottom']">(:obj <xsl:value-of select="panel[@position='bottom']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose>
	:west-attachment	(:point 0 0)</xsl:if>
<xsl:if test="$position='right'">	:north-attachment	<xsl:choose><xsl:when test="*[@position='top'] and not(panel[@position='top'])">(:obj <xsl:value-of select="@id"/>_top 0)</xsl:when><xsl:when test="panel[@position='top']">(:obj <xsl:value-of select="panel[@position='top']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose>
	:south-attachment	<xsl:choose><xsl:when test="*[@position='bottom'] and not(panel[@position='bottom'])">(:obj <xsl:value-of select="@id"/>_bottom 0)</xsl:when><xsl:when test="panel[@position='bottom']">(:obj <xsl:value-of select="panel[@position='bottom']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose>
	:east-attachment	(:point 0 0)</xsl:if>
<xsl:if test="$position='center'">	:north-attachment	<xsl:choose><xsl:when test="*[@position='top'] and not(panel[@position='top'])">(:obj <xsl:value-of select="@id"/>_top 0)</xsl:when><xsl:when test="panel[@position='top']">(:obj <xsl:value-of select="panel[@position='top']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose>
	:south-attachment	<xsl:choose><xsl:when test="*[@position='bottom'] and not(panel[@position='bottom'])">(:obj <xsl:value-of select="@id"/>_bottom 0)</xsl:when><xsl:when test="panel[@position='bottom']">(:obj <xsl:value-of select="panel[@position='bottom']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose>
	:west-attachment	<xsl:choose><xsl:when test="*[@position='left'] and not(panel[@position='left'])">(:obj <xsl:value-of select="@id"/>_left 0)</xsl:when><xsl:when test="panel[@position='left']">(:obj <xsl:value-of select="panel[@position='left']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose>
	:east-attachment	<xsl:choose><xsl:when test="*[@position='right'] and not(panel[@position='right'])">(:obj <xsl:value-of select="@id"/>_right 0)</xsl:when><xsl:when test="panel[@position='right']">(:obj <xsl:value-of select="panel[@position='right']/@id"/> 0)</xsl:when><xsl:otherwise>(:point 0 0)</xsl:otherwise></xsl:choose></xsl:if>
<xsl:if test="$position='middle'">	:north-attachment	(:center-grid-line 50 0)
	:west-attachment	(:center-grid-line 50 0)</xsl:if>
</xsl:otherwise></xsl:choose>
</xsl:template>

<!--
	insertlayoutpanels
	******************
	Inserts panels for the selected layout when needed.
-->
<xsl:template name="insertlayoutpanels">
<xsl:if test="(((@layout='horizontal' or @layout='vertical') and *) and not(local-name()='panel')) or (@layoutalign='middle')">
:element <xsl:value-of select="@id"/><xsl:choose><xsl:when test="@layoutalign='middle'">_middle</xsl:when><xsl:otherwise>_main</xsl:otherwise></xsl:choose>
(
	:type			:container
	:container-type		:relative
	:active			:true
	:visible		:true<xsl:if test="@popup"><xsl:text>
	</xsl:text>:menu			<xsl:value-of select="@popup"/></xsl:if>
	:border-frame		:none
<xsl:choose><xsl:when test="@layoutalign='middle'"><xsl:call-template name="insertpanelattachments"><xsl:with-param name="position" select="'middle'"/></xsl:call-template></xsl:when><xsl:otherwise><xsl:call-template name="insertpanelattachments"><xsl:with-param name="position" select="'center'"/></xsl:call-template></xsl:otherwise></xsl:choose>
        :children		(
<xsl:for-each select="button|statictext|input|checkbox|radiogroup|list|image|progressbar|scale|panel"><xsl:text>		</xsl:text><xsl:value-of select="@id"/><xsl:call-template name="insertpopupref"/><xsl:text>
</xsl:text></xsl:for-each>	)
)</xsl:if>
<xsl:if test="@layout='border'">
<xsl:if test="*[@position='top'] and not(panel[@position='top'])">
:element <xsl:value-of select="@id"/>_top
(
	:type			:container
	:container-type		:relative
	:active			:true
	:visible		:true<xsl:if test="@popup"><xsl:text>
	</xsl:text>:menu			<xsl:value-of select="@popup"/></xsl:if>
	:border-frame		:none
<xsl:call-template name="insertpanelattachments"><xsl:with-param name="position" select="'top'"/></xsl:call-template>
        :children		(
        	<xsl:value-of select="*[@position='top']/@id"/>
	)
)</xsl:if>
<xsl:if test="*[@position='bottom'] and not(panel[@position='bottom'])">
:element <xsl:value-of select="@id"/>_bottom
(
	:type			:container
	:container-type		:relative
	:active			:true
	:visible		:true<xsl:if test="@popup"><xsl:text>
	</xsl:text>:menu			<xsl:value-of select="@popup"/></xsl:if>
	:border-frame		:none
<xsl:call-template name="insertpanelattachments"><xsl:with-param name="position" select="'bottom'"/></xsl:call-template>
        :children		(
        	<xsl:value-of select="*[@position='bottom']/@id"/>
	)
)</xsl:if>
<xsl:if test="*[@position='left'] and not(panel[@position='left'])">
:element <xsl:value-of select="@id"/>_left
(
	:type			:container
	:container-type		:relative
	:active			:true
	:visible		:true<xsl:if test="@popup"><xsl:text>
	</xsl:text>:menu			<xsl:value-of select="@popup"/></xsl:if>
	:border-frame		:none
<xsl:call-template name="insertpanelattachments"><xsl:with-param name="position" select="'left'"/></xsl:call-template>
        :children		(
        	<xsl:value-of select="*[@position='left']/@id"/>
	)
)</xsl:if>
<xsl:if test="*[@position='right'] and not(panel[@position='right'])">
:element <xsl:value-of select="@id"/>_right
(
	:type			:container
	:container-type		:relative
	:active			:true
	:visible		:true<xsl:if test="@popup"><xsl:text>
	</xsl:text>:menu			<xsl:value-of select="@popup"/></xsl:if>
	:border-frame		:none
<xsl:call-template name="insertpanelattachments"><xsl:with-param name="position" select="'right'"/></xsl:call-template>
        :children		(
        	<xsl:value-of select="*[@position='right']/@id"/>
	)
)</xsl:if>
<xsl:if test="*[@position='center'] and not(panel[@position='center'])">
:element <xsl:value-of select="@id"/>_center
(
	:type			:container
	:container-type		:relative
	:active			:true
	:visible		:true<xsl:if test="@popup"><xsl:text>
	</xsl:text>:menu			<xsl:value-of select="@popup"/></xsl:if>
	:border-frame		:none
<xsl:call-template name="insertpanelattachments"><xsl:with-param name="position" select="'center'"/></xsl:call-template>
        :children		(
        	<xsl:value-of select="*[@position='center']/@id"/>
	)
)</xsl:if>
</xsl:if></xsl:template>

<!--
	insertlayoutrefs
	****************
	Inserts the names of panels to insert when needed (to use within the children-clause of a window or panel).
-->
<xsl:template name="insertlayoutrefs">
<xsl:if test="(@layout='horizontal' or @layout='vertical') and *"><xsl:text>
		</xsl:text><xsl:value-of select="@id"/><xsl:if test="not(local-name()='panel')">_main</xsl:if></xsl:if>
<xsl:if test="@layout='border'"><xsl:if test="*[@position='top']"><xsl:text>
		</xsl:text><xsl:choose><xsl:when test="panel[@position='top']"><xsl:value-of select="panel[@position='top']/@id"/></xsl:when><xsl:otherwise><xsl:value-of select="@id"/>_top</xsl:otherwise></xsl:choose></xsl:if>
		<xsl:if test="*[@position='bottom']"><xsl:text>
		</xsl:text><xsl:choose><xsl:when test="panel[@position='bottom']"><xsl:value-of select="panel[@position='bottom']/@id"/></xsl:when><xsl:otherwise><xsl:value-of select="@id"/>_bottom</xsl:otherwise></xsl:choose></xsl:if>
		<xsl:if test="*[@position='left']"><xsl:text>
		</xsl:text><xsl:choose><xsl:when test="panel[@position='left']"><xsl:value-of select="panel[@position='left']/@id"/></xsl:when><xsl:otherwise><xsl:value-of select="@id"/>_left</xsl:otherwise></xsl:choose></xsl:if>
		<xsl:if test="*[@position='right']"><xsl:text>
		</xsl:text><xsl:choose><xsl:when test="panel[@position='right']"><xsl:value-of select="panel[@position='right']/@id"/></xsl:when><xsl:otherwise><xsl:value-of select="@id"/>_right</xsl:otherwise></xsl:choose></xsl:if>
		<xsl:if test="*[@position='center']"><xsl:text>
		</xsl:text><xsl:choose><xsl:when test="panel[@position='center']"><xsl:value-of select="panel[@position='center']/@id"/></xsl:when><xsl:otherwise><xsl:value-of select="@id"/>_center</xsl:otherwise></xsl:choose></xsl:if></xsl:if>
</xsl:template>

<!--
	insertpopupref
	**************
	Inserts a reference to a popup panel when a popup menu is specified.
-->
<xsl:template name="insertpopupref"><xsl:if test="@popup">_popup</xsl:if></xsl:template>

<!--
	insertpopuppanel
	****************
	Inserts a popup panel when a popup menu is specified.
-->
<xsl:template name="insertpopuppanel">
<xsl:if test="@popup">
:element <xsl:value-of select="@id"/>_popup
(
	:type			:container
	:container-type		:relative
	:active			:true
	:visible		:true
	:menu			<xsl:value-of select="@popup"/>
	:border-frame		:none
<xsl:call-template name="insertattachments"/>
        :children		(
        	<xsl:value-of select="@id"/>
	)
)</xsl:if></xsl:template>

<!--
	insertpopupattachments
	**********************
	Inserts attachments to the parent popup panel.
-->
<xsl:template name="insertpopupattachments">
	:north-attachment	(:point 0 0)
	:south-attachment	(:point 0 0)
	:east-attachment	(:point 0 0)
	:west-attachment	(:point 0 0)
</xsl:template>

<!--
	insertcolors
	************
	Inserts colors when specified.
-->
<xsl:template name="insertcolors">
<xsl:if test="@foregroundcolor"><xsl:text>
	</xsl:text>:fg-color		"<xsl:value-of select="@foregroundcolor"/>"</xsl:if>
<xsl:if test="@backgroundcolor"><xsl:text>
	</xsl:text>:bg-color		"<xsl:value-of select="@backgroundcolor"/>"</xsl:if>
</xsl:template>

<!--
	insertconnections
	*****************
	Inserts the specified connections.
-->
<xsl:template name="insertconnections"><xsl:with-param name="menucheck"/><!-- Needed for implementing checkable menuitems -->
<xsl:if test="@oncreate='true'">
:connection
(
	:from			<xsl:value-of select="@id"/>
	:when			:after-create
	:action-type		:execute-code
	:action			"<xsl:element name="xsl:value-of"><xsl:attribute name="disable-output-escaping">yes</xsl:attribute><xsl:attribute name="select">code[@id='<xsl:value-of select="@id"/>' and @event='oncreate']</xsl:attribute></xsl:element>"
)</xsl:if>
<xsl:if test="@ondestroy='true'">
:connection
(
	:from			<xsl:value-of select="@id"/>
	:when			:destroy
	:action-type		:execute-code
	:action			"<xsl:element name="xsl:value-of"><xsl:attribute name="disable-output-escaping">yes</xsl:attribute><xsl:attribute name="select">code[@id='<xsl:value-of select="@id"/>' and @event='oncreate']</xsl:attribute></xsl:element>"
)</xsl:if>
<xsl:if test="@onshow='true'">
:connection
(
	:from			<xsl:value-of select="@id"/>
	:when			:<xsl:choose><xsl:when test="local-name()='popupmenu'">popup</xsl:when><xsl:otherwise>show</xsl:otherwise></xsl:choose>
	:action-type		:execute-code
	:action			"<xsl:element name="xsl:value-of"><xsl:attribute name="disable-output-escaping">yes</xsl:attribute><xsl:attribute name="select">code[@id='<xsl:value-of select="@id"/>' and @event='onshow']</xsl:attribute></xsl:element>"
)</xsl:if>
<xsl:if test="@onhide='true'">
:connection
(
	:from			<xsl:value-of select="@id"/>
	:when			:hide
	:action-type		:execute-code
	:action			"<xsl:element name="xsl:value-of"><xsl:attribute name="disable-output-escaping">yes</xsl:attribute><xsl:attribute name="select">code[@id='<xsl:value-of select="@id"/>' and @event='onhide']</xsl:attribute></xsl:element>"
)</xsl:if>
<xsl:if test="@onclick='true'">
:connection
(
	:from			<xsl:value-of select="@id"/><xsl:choose><xsl:when test="local-name()='checkbox'">_item</xsl:when><xsl:when test="local-name()='menu'">_menu</xsl:when><xsl:when test="@checkable='true'">_<xsl:value-of select="$menucheck"/></xsl:when></xsl:choose>
	:when			:<xsl:choose><xsl:when test="local-name()='checkbox'">toggle</xsl:when><xsl:when test="local-name='menu'">popup</xsl:when><xsl:otherwise>activate</xsl:otherwise></xsl:choose>
	:action-type		:execute-code
	:action			"<xsl:element name="xsl:value-of"><xsl:attribute name="disable-output-escaping">yes</xsl:attribute><xsl:attribute name="select">code[@id='<xsl:value-of select="@id"/>' and @event='onclick']</xsl:attribute></xsl:element>"
)</xsl:if>
<xsl:if test="@onchange='true' and not(local-name()='radiogroup')">
:connection
(
	:from			<xsl:value-of select="@id"/>
	:when			:<xsl:choose><xsl:when test="local-name()='scale'">value-changed</xsl:when><xsl:otherwise>text-changed</xsl:otherwise></xsl:choose>
	:action-type		:execute-code
	:action			"<xsl:element name="xsl:value-of"><xsl:attribute name="disable-output-escaping">yes</xsl:attribute><xsl:attribute name="select">code[@id='<xsl:value-of select="@id"/>' and @event='onchange']</xsl:attribute></xsl:element>"
)</xsl:if>
<xsl:if test="@onselect='true'">
:connection
(
	:from			<xsl:value-of select="@id"/>
	:when			:item-selected
	:action-type		:execute-code
	:action			"<xsl:element name="xsl:value-of"><xsl:attribute name="disable-output-escaping">yes</xsl:attribute><xsl:attribute name="select">code[@id='<xsl:value-of select="@id"/>' and @event='onselect']</xsl:attribute></xsl:element>"
)</xsl:if>
</xsl:template>

<!--
	ELEMENT SPECIFIC TEMPLATES
	**************************************************
-->
<!--
	window
	******
-->
<xsl:template match="window">
:module mod_<xsl:value-of select="@id"/>
(
<xsl:call-template name="insertheader"/>
	:type			:base-window
	:bg-color		"white"<xsl:if test="@image"><xsl:text>
</xsl:text>	:icon-file		"<xsl:value-of select="@image"/>"</xsl:if>
	:icon-label		"<xsl:value-of select="@text"/>"
	:label			"<xsl:value-of select="@text"/>"
	:resizable		:<xsl:value-of select="@resizable"/>
	:iconic			:false
	:visible		:<xsl:value-of select="@visible"/>
	:children		(<xsl:if test="menubar"><xsl:text>
		</xsl:text><xsl:value-of select="menubar[position()=1]/@id"/></xsl:if>
		<xsl:choose><xsl:when test="@layoutalign='middle'"><xsl:value-of select="@id"/>_main</xsl:when><xsl:otherwise><xsl:call-template name="insertlayoutrefs"/></xsl:otherwise></xsl:choose>
	)
)
<xsl:if test="@layoutalign='middle'">:element <xsl:value-of select="@id"/>_main
(
	:type			:container
	:container-type		:relative
	:active			:true
	:visible		:true<xsl:if test="@popup"><xsl:text>
	</xsl:text>:menu			<xsl:value-of select="@popup"/></xsl:if>
	:border-frame		:none
<xsl:call-template name="insertpanelattachments"><xsl:with-param name="position" select="'center'"/></xsl:call-template>
        :children		(
		<xsl:value-of select="@id"/>_middle
	)
)</xsl:if>
<xsl:apply-templates select="menubar[position()=1]"/>
<xsl:call-template name="insertlayoutpanels"/>
<xsl:apply-templates select="button|statictext|input|checkbox|radiogroup|popupmenu|list|panel|image|progressbar|scale"/>
<xsl:call-template name="insertconnections"/>

)
</xsl:template>

<!--
	button
	******
-->
<xsl:template match="button">
<xsl:call-template name="insertpopuppanel"/>
<xsl:call-template name="insertheader"/>
	:type			:button
	:button-type		:push-button
	:label-type		:string
	:label			"<xsl:value-of select="@text"/>"<xsl:call-template name="insertcolors"/>
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
	:visible		:true
<xsl:choose><xsl:when test="@popup"><xsl:call-template name="insertpopupattachments"/></xsl:when><xsl:otherwise><xsl:call-template name="insertattachments"/></xsl:otherwise></xsl:choose>
)<xsl:call-template name="insertconnections"/></xsl:template>

<!--
	statictext
	**********
-->
<xsl:template match="statictext">
<xsl:call-template name="insertpopuppanel"/>
<xsl:call-template name="insertheader"/>
	:type			:label
	:label-type		:string
	:label			"<xsl:value-of select="normalize-space(line[position()=1])"/><xsl:for-each select="line[position()>1]">\n<xsl:value-of select="normalize-space(text())"/></xsl:for-each>"
	:label-alignment	:left<xsl:call-template name="insertcolors"/>
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
	:visible		:true
<xsl:choose><xsl:when test="@popup"><xsl:call-template name="insertpopupattachments"/></xsl:when><xsl:otherwise><xsl:call-template name="insertattachments"/></xsl:otherwise></xsl:choose>
)<xsl:call-template name="insertconnections"/></xsl:template>

<!--
	input
	*****
-->
<xsl:template match="input">
<xsl:call-template name="insertpopuppanel"/>
<xsl:call-template name="insertheader"/>
<xsl:choose><xsl:when test="@numberoflines = '1'">
	:type			:text-field
	:text-type		:alphanumeric<xsl:if test="@charsonline"><xsl:text>
	</xsl:text>:num-columns		<xsl:value-of select="@charsonline"/></xsl:if>
	:initial-value		"<xsl:value-of select="normalize-space(line[position()=1])"/>"<xsl:call-template name="insertcolors"/>
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
	:visible		:true
</xsl:when><xsl:otherwise>
	:type			:text-pane
	:text-type		:alphanumeric<xsl:if test="@charsonline"><xsl:text>
	</xsl:text>:num-columns		<xsl:value-of select="@charsonline"/></xsl:if>
	:num-rows		<xsl:value-of select="@numberoflines"/>
	:word-wrap		:<xsl:value-of select="@wordwrap"/>
	:initial-value		"<xsl:value-of select="normalize-space(line[position()=1])"/><xsl:for-each select="line[position()>1]">\n<xsl:value-of select="normalize-space(text())"/></xsl:for-each>"<xsl:call-template name="insertcolors"/>
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
	:visible		:true<xsl:choose><xsl:when test="@scrollbar = 'none'">
	:hscrollbar		:never
	:vscrollbar		:never</xsl:when><xsl:when test="@scrollbar = 'vertical'">
	:hscrollbar		:never
	:vscrollbar		:always</xsl:when><xsl:when test="@scrollbar = 'horizontal'">
	:hscrollbar		:always
	:vscrollbar		:never</xsl:when><xsl:otherwise>
	:hscrollbar		:always
	:vscrollbar		:always</xsl:otherwise></xsl:choose></xsl:otherwise></xsl:choose><xsl:text>
</xsl:text><xsl:choose><xsl:when test="@popup"><xsl:call-template name="insertpopupattachments"/></xsl:when><xsl:otherwise><xsl:call-template name="insertattachments"/></xsl:otherwise></xsl:choose>
)<xsl:call-template name="insertconnections"/></xsl:template>

<!--
	menubar
	*******
-->
<xsl:template match="menubar">
<xsl:call-template name="insertheader"/>
	:type			:container
	:container-type		:menu-bar<xsl:call-template name="insertcolors"/>
        :children		(
<xsl:for-each select="menuitem|menuseparator|menu"><xsl:text>		</xsl:text><xsl:value-of select="@id"/><xsl:text>
</xsl:text></xsl:for-each>	)
)<xsl:apply-templates select="menuitem|menuseparator|menu"/>
<xsl:call-template name="insertconnections"/></xsl:template>

<!--
	menuitem
	********
-->
<xsl:template name="dtbname"><xsl:value-of select="concat(translate(substring(/window/@id,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'),substring(/window/@id,2))"/></xsl:template>
<xsl:template name="dtb_name"><xsl:value-of select="/window/@id"/>_<xsl:value-of select="/window/@id"/></xsl:template>

<xsl:template name="menunum"><xsl:with-param name="num"/>
<xsl:for-each select="parent::*"><!-- OK, this is a bit of a hack (technology :-), but AFAIK the easiest way to change the current node up to the parent node -->
<xsl:choose><xsl:when test="local-name(parent::*) = 'menubar'"><xsl:if test="$num != '1'"><xsl:value-of select="$num"/></xsl:if></xsl:when>
<xsl:otherwise><xsl:call-template name="menunum"><xsl:with-param name="num" select="$num + 1"/></xsl:call-template></xsl:otherwise></xsl:choose>
</xsl:for-each>

</xsl:template>

<xsl:template name="menuitemcheckcode"><xsl:with-param name="menucheck"/><!-- Needed for implementing checkable menuitems --><xsl:element name="xsl:text"><xsl:attribute name="disable-output-escaping">yes</xsl:attribute>
<xsl:text>/*** This code implements toggle behaviour for this menuitem. ***/
    /*** DO NOT CHANGE THE CODE BETWEEN 'BEGIN CODE' AND 'END CODE'! ***/
    /*** BEGIN CODE vvv ***/
    DtbMod</xsl:text><xsl:call-template name="dtbname"/><xsl:call-template name="dtbname"/><xsl:text>Info	dtbTarget = (DtbMod</xsl:text><xsl:call-template name="dtbname"/><xsl:call-template name="dtbname"/><xsl:text>Info)clientData;
    DtbMod</xsl:text><xsl:call-template name="dtbname"/><xsl:call-template name="dtbname"/><xsl:text>Info	instance = dtbTarget;	/* obsolete */
    
    if (!(dtbTarget->initialized))
    {
        dtb_mod_</xsl:text><xsl:call-template name="dtb_name"/><xsl:text>_initialize(dtbTarget, dtb_get_toplevel_widget());
    }
    
    XtManageChild(dtb_mod_</xsl:text><xsl:call-template name="dtb_name"/>.<xsl:value-of select="../@id"/>_<xsl:value-of select="../@id"/>_menu<xsl:call-template name="menunum"><xsl:with-param name="num" select="'1'"/></xsl:call-template>_items.<xsl:value-of select="@id"/>_<xsl:choose><xsl:when test="$menucheck='on'">off</xsl:when><xsl:otherwise>on</xsl:otherwise></xsl:choose><xsl:text>);
    XtUnmanageChild(dtb_mod_</xsl:text><xsl:call-template name="dtb_name"/>.<xsl:value-of select="../@id"/>_<xsl:value-of select="../@id"/>_menu<xsl:call-template name="menunum"><xsl:with-param name="num" select="'1'"/></xsl:call-template>_items.<xsl:value-of select="@id"/>_<xsl:choose><xsl:when test="$menucheck='on'">on</xsl:when><xsl:otherwise>off</xsl:otherwise></xsl:choose><xsl:text>);
    /*** END CODE   ^^^ ***/
</xsl:text></xsl:element></xsl:template>

<xsl:template match="menuitem">
<xsl:choose><xsl:when test="@checkable='false'">
<xsl:call-template name="insertheader"/>
	:type			:item
	:item-type		:item-for-menu
	:label-type		:string
	:label			"   <xsl:value-of select="@text"/>   "<xsl:call-template name="insertcolors"/>
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
)<xsl:call-template name="insertconnections"/>
</xsl:when><xsl:otherwise>
:element <xsl:value-of select="@id"/>_on
(
	:type			:item
	:item-type		:item-for-menu
	:label-type		:string
	:label			"x <xsl:value-of select="@text"/>   "<xsl:call-template name="insertcolors"/>
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
	:visible		:<xsl:choose><xsl:when test="@checked = 'true'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
)
:connection
(
	:from			<xsl:value-of select="@id"/>_on
	:when			:activate
	:action-type		:execute-code
	:action			"<xsl:call-template name="menuitemcheckcode"><xsl:with-param name="menucheck" select="'on'"/></xsl:call-template>"
)<xsl:call-template name="insertconnections"><xsl:with-param name="menucheck" select="'on'"/></xsl:call-template><!-- Needed for implementing checkable menuitems -->
:element <xsl:value-of select="@id"/>_off
(
	:type			:item
	:item-type		:item-for-menu
	:label-type		:string
	:label			"   <xsl:value-of select="@text"/>   "<xsl:call-template name="insertcolors"/>
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
	:visible		:<xsl:choose><xsl:when test="@checked = 'false'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
)
:connection
(
	:from			<xsl:value-of select="@id"/>_off
	:when			:activate
	:action-type		:execute-code
	:action			"<xsl:call-template name="menuitemcheckcode"><xsl:with-param name="menucheck" select="'off'"/></xsl:call-template>"
)<xsl:call-template name="insertconnections"><xsl:with-param name="menucheck" select="'off'"/></xsl:call-template><!-- Needed for implementing checkable menuitems -->
</xsl:otherwise></xsl:choose></xsl:template>

<!--
	menuseparator
	*************
-->
<xsl:template match="menuseparator">
<xsl:call-template name="insertheader"/>
	:type			:item
	:item-type		:item-for-menu
	:label-type		:separator
	:line-style		:etched-in<xsl:call-template name="insertcolors"/>
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
)<xsl:call-template name="insertconnections"/></xsl:template>

<!--
	menu
	****
-->
<xsl:template match="menu">
<xsl:call-template name="insertheader"/>
	:type			:item
	:item-type		:item-for-menubar
	:label-type		:string
	:label			"<xsl:if test="local-name(parent::*) != 'menubar'"><xsl:text>   </xsl:text></xsl:if><xsl:value-of select="@text"/><xsl:if test="local-name(parent::*) != 'menubar'"><xsl:text>   </xsl:text></xsl:if>"<xsl:call-template name="insertcolors"/>
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
	:menu			<xsl:value-of select="@id"/>_menu
)
:element <xsl:value-of select="@id"/>_menu
(
	:type			:menu<xsl:call-template name="insertcolors"/>
	:tear-off		:<xsl:value-of select="@tearoff"/>
        :children		(
<xsl:for-each select="menuitem|menuseparator|menu"><xsl:text>		</xsl:text><xsl:value-of select="@id"/><xsl:if test="@checkable='true'">_on
		<xsl:value-of select="@id"/>_off</xsl:if><xsl:text>
</xsl:text></xsl:for-each>	)
)<xsl:apply-templates select="menuitem|menuseparator|menu"/>
<xsl:call-template name="insertconnections"/></xsl:template>

<!--
	checkbox
	********
-->
<xsl:template match="checkbox">
<xsl:call-template name="insertpopuppanel"/>
<xsl:call-template name="insertheader"/>
	:type			:choice
	:choice-type		:nonexclusive
	:num-columns		1
	:orientation		:horizontal
	:selection-required	:false<xsl:call-template name="insertcolors"/>
	:active			:true
	:visible		:true
<xsl:choose><xsl:when test="@popup"><xsl:call-template name="insertpopupattachments"/></xsl:when><xsl:otherwise><xsl:call-template name="insertattachments"/></xsl:otherwise></xsl:choose>
	:children		(
		<xsl:value-of select="@id"/>_item
	)
)
:element <xsl:value-of select="@id"/>_item
(
	:type			:item
	:item-type		:item-for-choice
	:label-type		:string
	:label			"<xsl:value-of select="@text"/>"<xsl:call-template name="insertcolors"/>
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
	:selected		:<xsl:value-of select="@checked"/>
)<xsl:call-template name="insertconnections"/></xsl:template>

<!--
	radiogroup
	**********
-->
<xsl:template match="radiogroup">
<xsl:call-template name="insertpopuppanel"/>
<xsl:call-template name="insertheader"/>
	:type			:choice
	:choice-type		:exclusive
	:num-columns		1
	:orientation		:vertical
	:selection-required	:false
	:label-type		:string
	:label			"<xsl:value-of select="@text"/>"
	:label-position		:north<xsl:call-template name="insertcolors"/>
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
	:visible		:true
<xsl:choose><xsl:when test="@popup"><xsl:call-template name="insertpopupattachments"/></xsl:when><xsl:otherwise><xsl:call-template name="insertattachments"/></xsl:otherwise></xsl:choose>
        :children		(
<xsl:for-each select="radioitem"><xsl:text>		</xsl:text><xsl:value-of select="@id"/><xsl:text>
</xsl:text></xsl:for-each>	)
)<xsl:apply-templates select="radioitem"/>
<xsl:call-template name="insertconnections"/></xsl:template>

<!--
	radioitem
	*********
-->
<xsl:template match="radioitem">
<xsl:call-template name="insertheader"/>
	:type			:item
	:item-type		:item-for-choice
	:label-type		:string
	:label			"<xsl:value-of select="@text"/>"
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
	:selected		:<xsl:value-of select="@checked"/>
)
:connection
(
	:from			<xsl:value-of select="@id"/>
	:when			:activate
	:action-type		:execute-code
	:action			"<xsl:element name="xsl:value-of"><xsl:attribute name="select">normalize-space(code[@id='<xsl:value-of select="../@id"/>' and @event='onchange'])</xsl:attribute></xsl:element>"
)<xsl:call-template name="insertconnections"/></xsl:template>

<!--
	popupmenu
	*********
-->
<xsl:template match="popupmenu">
<xsl:call-template name="insertheader"/>
	:type			:menu<xsl:call-template name="insertcolors"/>
	:tear-off		:<xsl:value-of select="@tearoff"/>
        :children		(
<xsl:for-each select="menuitem|menu"><xsl:text>		</xsl:text><xsl:value-of select="@id"/><xsl:text>
</xsl:text></xsl:for-each>	)
)<xsl:apply-templates select="menuitem|menu"/>
<xsl:call-template name="insertconnections"/></xsl:template>

<!--
	list
	****
-->
<xsl:template match="list">
<xsl:call-template name="insertpopuppanel"/>
<xsl:call-template name="insertheader"/>
	:type			:list
	:selection-mode		:<xsl:choose><xsl:when test="@selectmode = 'single'">browse</xsl:when><xsl:otherwise>browse-multiple</xsl:otherwise></xsl:choose>
	:selection-required	:false<xsl:if test="@numberoflines"><xsl:text>
	</xsl:text>:num-rows		<xsl:value-of select="@numberoflines"/></xsl:if><xsl:call-template name="insertcolors"/>
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
	:visible		:true
<xsl:choose><xsl:when test="@popup"><xsl:call-template name="insertpopupattachments"/></xsl:when><xsl:otherwise><xsl:call-template name="insertattachments"/></xsl:otherwise></xsl:choose>
        :children		(
<xsl:for-each select="line"><xsl:text>		</xsl:text><xsl:value-of select="../@id"/>_item<xsl:value-of select="position()"/><xsl:text>
</xsl:text></xsl:for-each>	)
)<xsl:for-each select="line">
:element <xsl:value-of select="../@id"/>_item<xsl:value-of select="position()"/>
(
	:type			:item
	:item-type		:item-for-list
	:label			"<xsl:value-of select="normalize-space(text())"/>"
	:selected		:false
)</xsl:for-each>
<xsl:call-template name="insertconnections"/></xsl:template>

<!--
	panel
	*****
-->
<xsl:template match="panel">
<xsl:call-template name="insertheader"/>
	:type			:container
	:container-type		:relative<xsl:call-template name="insertcolors"/>
	:active			:true
	:visible		:true<xsl:if test="@popup"><xsl:text>
	</xsl:text>:menu			<xsl:value-of select="@popup"/></xsl:if>
	:border-frame		:<xsl:choose><xsl:when test="@bevel='flat' and @border='flat'">none</xsl:when><xsl:when test="@bevel='flat' and @border='sunken'">etched-in</xsl:when><xsl:when test="@bevel='flat' and @border='raised'">etched-out</xsl:when><xsl:when test="@bevel='sunken'">shadow-in</xsl:when><xsl:when test="@bevel='raised'">shadow-out</xsl:when></xsl:choose><xsl:text>
</xsl:text><xsl:call-template name="insertattachments"/>
	:children		(<xsl:choose><xsl:when test="@layout='border'"><xsl:call-template name="insertlayoutrefs"/></xsl:when><xsl:when test="@layoutalign='middle'"><xsl:text>
		</xsl:text><xsl:value-of select="@id"/>_middle</xsl:when><xsl:otherwise><xsl:text>
</xsl:text><xsl:for-each select="button|statictext|input|checkbox|radiogroup|popupmenu|list|image|progressbar|scale|panel"><xsl:text>		</xsl:text><xsl:value-of select="@id"/><xsl:text>
</xsl:text></xsl:for-each></xsl:otherwise></xsl:choose>
	)
)<xsl:call-template name="insertlayoutpanels"/>
<xsl:apply-templates select="button|statictext|input|checkbox|radiogroup|list|panel|image|progressbar|scale"/>
<xsl:call-template name="insertconnections"/></xsl:template>

<!--
	image
	*****
-->
<xsl:template match="image">
<xsl:call-template name="insertpopuppanel"/>
<xsl:call-template name="insertheader"/>
	:type			:label
	:label-type		:graphic
	:label			"<xsl:value-of select="@source"/>.gif"<xsl:call-template name="insertcolors"/>
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
	:visible		:true
<xsl:choose><xsl:when test="@popup"><xsl:call-template name="insertpopupattachments"/></xsl:when><xsl:otherwise><xsl:call-template name="insertattachments"/></xsl:otherwise></xsl:choose>
)<xsl:call-template name="insertconnections"/></xsl:template>
	
<!--
	progressbar/scale
	*****************
-->
<xsl:template match="progressbar|scale">
<xsl:call-template name="insertpopuppanel"/>
<xsl:call-template name="insertheader"/>
	:type			:scale
	:read-only		:<xsl:choose><xsl:when test="local-name()='scale'">false</xsl:when>
<xsl:otherwise>true</xsl:otherwise></xsl:choose>
	:orientation		:<xsl:value-of select="@orientation"/>
	:direction		:<xsl:if test="@orientation='horizontal'">left-to-right</xsl:if><xsl:if test="@orientation='vertical'">bottom-to-top</xsl:if>
	:min-value		<xsl:value-of select="@min"/>
	:max-value		<xsl:value-of select="@max"/>
	:initial-value		<xsl:value-of select="@position"/>
	:increment		<xsl:choose><xsl:when test="local-name()='scale'"><xsl:value-of select="@increment"/></xsl:when>
<xsl:otherwise>1</xsl:otherwise></xsl:choose>
	:decimal-points		0
	:show-value		:<xsl:value-of select="@showvalue"/><xsl:call-template name="insertcolors"/>
	:active			:<xsl:choose><xsl:when test="@state = 'enabled'">true</xsl:when><xsl:otherwise>false</xsl:otherwise></xsl:choose>
	:visible		:true
<xsl:choose><xsl:when test="@popup"><xsl:call-template name="insertpopupattachments"/></xsl:when><xsl:otherwise><xsl:call-template name="insertattachments"/></xsl:otherwise></xsl:choose>
)<xsl:call-template name="insertconnections"/></xsl:template>

</xsl:stylesheet>